@@ -6,7 +6,7 @@ else:
     from .util import secp256k1
 import hashlib
 from . import ec
-from .base import EmbitKey, EmbitError
+from .base import EmbitKey, EmbitError, copy
 from .networks import NETWORKS
 from . import base58
 from . import hashes
@@ -97,7 +97,12 @@ class HDKey(EmbitKey):
 
     def to_base58(self, version=None) -> str:
         b = self.serialize(version)
-        return base58.encode_check(b)
+        res = base58.encode_check(b)
+        if res[1:4] == "prv" and not self.is_private:
+            raise HDError("Invalid version for private key")
+        if res[1:4] == "pub" and self.is_private:
+            raise HDError("Invalid version for public key")
+        return res
 
     @classmethod
     def from_string(cls, s: str):
@@ -163,6 +168,19 @@ class HDKey(EmbitKey):
         """Returns SEC serialization of the public key"""
         return self.key.sec()
 
+    def xonly(self) -> bytes:
+        return self.key.xonly()
+
+    def taproot_tweak(self, h=b""):
+        return HDKey(
+            self.key.taproot_tweak(h),
+            self.chain_code,
+            version=self.version,
+            depth=self.depth,
+            fingerprint=self.fingerprint,
+            child_number=self.child_number,
+        )
+
     def child(self, index: int, hardened: bool = False):
         """Derives a child HDKey"""
         if index > 0xFFFFFFFF:
@@ -189,13 +207,13 @@ class HDKey(EmbitKey):
             key = ec.PrivateKey(secret)
         else:
             # copy of internal secp256k1 point structure
-            point = self.key._point[:]
+            point = copy(self.key._point)
             point = secp256k1.ec_pubkey_add(point, secret)
             key = ec.PublicKey(point)
         return HDKey(
             key,
             chain_code,
-            version=self.version[:],
+            version=self.version,
             depth=self.depth + 1,
             fingerprint=fingerprint,
             child_number=index,
@@ -217,12 +235,16 @@ class HDKey(EmbitKey):
             raise HDError("HD public key can't sign")
         return self.key.sign(msg_hash)
 
-    def verify(self, sig: ec.Signature, msg_hash: bytes) -> bool:
-        """Verifies a signature agains 32-byte message hash"""
-        if self.is_private:
-            return self.key.get_public_key().verify(sig, msg_hash)
-        else:
-            return self.key.verify(sig, msg_hash)
+    def schnorr_sign(self, msg_hash):
+        if not self.is_private:
+            raise HDError("HD public key can't sign")
+        return self.key.schnorr_sign(msg_hash)
+
+    def verify(self, sig, msg_hash) -> bool:
+        return self.key.verify(sig, msg_hash)
+
+    def schnorr_verify(self, sig, msg_hash) -> bool:
+        return self.key.schnorr_verify(sig, msg_hash)
 
     def __eq__(self, other):
         # skip version
@@ -232,29 +254,30 @@ class HDKey(EmbitKey):
         return hash(self.serialize())
 
 
-def detect_version(path: str, default="xprv", network=None) -> bytes:
+def detect_version(path, default="xprv", network=None) -> bytes:
     """
-    Detects slip-132? version from the path for certain network.
+    Detects slip-132 version from the path for certain network.
     Trying to be smart, use if you want, but with care.
     """
     key = default
     net = network
     if network is None:
         net = NETWORKS["main"]
-    arr = parse_path(path)
-    if len(arr) == 0:
+    if isinstance(path, str):
+        path = parse_path(path)
+    if len(path) == 0:
         return network[key]
-    if arr[0] == 0x80000000 + 84:
+    if path[0] == 0x80000000 + 84:
         key = "z" + default[1:]
-    elif arr[0] == 0x80000000 + 49:
+    elif path[0] == 0x80000000 + 49:
         key = "y" + default[1:]
-    elif arr[0] == 0x80000000 + 48:
-        if len(arr) >= 4:
-            if arr[3] == 0x80000000 + 1:
+    elif path[0] == 0x80000000 + 48:
+        if len(path) >= 4:
+            if path[3] == 0x80000000 + 1:
                 key = "Y" + default[1:]
-            elif arr[3] == 0x80000000 + 2:
+            elif path[3] == 0x80000000 + 2:
                 key = "Z" + default[1:]
-    if network is None and len(arr) > 1 and arr[1] == 0x80000000 + 1:
+    if network is None and len(path) > 1 and path[1] == 0x80000000 + 1:
         net = NETWORKS["test"]
     return net[key]
 
